@@ -1,5 +1,5 @@
 // Preferences.cc - source file for the mailfilter program
-// Copyright (c) 2000 - 2002  Andreas Bauer <baueran@in.tum.de>
+// Copyright (c) 2000 - 2004  Andreas Bauer <baueran@in.tum.de>
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -41,6 +41,7 @@ namespace pref {
   Preferences::Preferences(const string& rcfileName) {
     // Set the program's default parameters
     file = rcfileName;
+    highscore = 100;
     maxsize = 0;
     maxsizeFriends = 0;
     icase = true;
@@ -54,6 +55,8 @@ namespace pref {
     showHeaders = false;
     delDuplicates = false;
     compiled = false;     // Flag, indicating whether the filters are already pre-compiled (or not)
+    size_score.score = 0;
+    size_score.size = 0;
   }
   
   
@@ -61,6 +64,7 @@ namespace pref {
     // Now clean up the memory used by Regular Expressions
     vector<filterInfo>::iterator curFilter = getFilters()->begin();
     vector<friendInfo>::iterator curFriend = getFriends()->begin();
+    vector<scoreInfo>::iterator curScore = getScores()->begin();
 
     // Only try to free memory of filters, if they were pre-compiled before (otherwise we're having a nice crash here...)
     if (compiled) {
@@ -73,12 +77,18 @@ namespace pref {
 	regExp.free(&curFriend->cFilter);
 	curFriend++;
       }
+
+      while (curScore != getScores()->end()) {
+	regExp.free(&curScore->cFilter);
+	curScore++;
+      }      
     }
 
     // Clean up the rest...
     servers.clear();
     filters.clear();
     friends.clear();
+    scores.clear();
   }
   
   
@@ -114,6 +124,8 @@ namespace pref {
 	popServer.protocol = POP3;
       else if (cmpNoCase(ident, "imap") == 0)
 	popServer.protocol = IMAP;
+      else if (cmpNoCase(ident, "apop") == 0)
+	popServer.protocol = APOP;
       else {
 	showErrorParameter(keyw, currOp, ident);
 	throw MalformedPrefsFile();
@@ -170,14 +182,12 @@ namespace pref {
 	newFilter.filter = ident;
 	newFilter.case_sensitive = CASE_DEFAULT;
 	newFilter.negative = true;
-	newFilter.matched = false;
 	filters.push_back(newFilter);
       }
       else {
 	newFilter.filter = ident;
 	newFilter.case_sensitive = CASE_DEFAULT;
 	newFilter.negative = false;
-	newFilter.matched = false;
 	filters.push_back(newFilter);
       }
     }
@@ -186,14 +196,12 @@ namespace pref {
 	newFilter.filter = ident;
 	newFilter.case_sensitive = CASE_INSENSITIVE;
 	newFilter.negative = true;
-	newFilter.matched = false;
 	filters.push_back(newFilter);
       }
       else {
 	newFilter.filter = ident;
 	newFilter.case_sensitive = CASE_INSENSITIVE;
 	newFilter.negative = false;
-	newFilter.matched = false;
 	filters.push_back(newFilter);
       }
     }
@@ -202,14 +210,12 @@ namespace pref {
 	newFilter.filter = ident;
 	newFilter.case_sensitive = CASE_SENSITIVE;
 	newFilter.negative = true;
-	newFilter.matched = false;
 	filters.push_back(newFilter);
       }
       else {
 	newFilter.filter = ident;
 	newFilter.case_sensitive = CASE_SENSITIVE;
 	newFilter.negative = false;
-	newFilter.matched = false;
 	filters.push_back(newFilter);
       }
     }
@@ -260,6 +266,8 @@ namespace pref {
 	popServer.port = ident;
 	servers.push_back(popServer);              // Always push here, cause after this either another server gets configured, or we're done
       }
+      else if (cmpNoCase(keyw, "HIGHSCORE") == 0)
+	highscore = ident;
       else if (cmpNoCase(keyw, "MAXSIZE_DENY") == 0)
 	maxsize = ident;
       else if (cmpNoCase(keyw, "MAXSIZE_ALLOW") == 0)
@@ -282,12 +290,82 @@ namespace pref {
       throw MalformedPrefsFile();
     }
   }
- 
+
+
+  void Preferences::keyword(const string keyw, int score, const string currOp, const string ident) {
+    scoreInfo newScore;
+    
+    if (cmpNoCase(keyw, "SCORE") == 0) {
+      if (currOp == "=") {
+	newScore.score = score;
+	newScore.filter = ident;
+	newScore.case_sensitive = CASE_DEFAULT;
+	newScore.negative = false;
+	scores.push_back(newScore);
+      }
+      else {
+	newScore.score = score;
+	newScore.filter = ident;
+	newScore.case_sensitive = CASE_DEFAULT;
+	newScore.negative = true;
+	scores.push_back(newScore);
+      }
+    }
+    else if (cmpNoCase(keyw, "SCORE_CASE") == 0) {
+      if (currOp == "=") {
+	newScore.score = score;
+	newScore.filter = ident;
+	newScore.case_sensitive = CASE_SENSITIVE;
+	newScore.negative = false;
+	scores.push_back(newScore);
+      }
+      else {
+	newScore.score = score;
+	newScore.filter = ident;
+	newScore.case_sensitive = CASE_SENSITIVE;
+	newScore.negative = true;
+	scores.push_back(newScore);
+      }
+    }
+    else if (cmpNoCase(keyw, "SCORE_NOCASE") == 0) {
+      if (currOp == "=") {
+	newScore.score = score;
+	newScore.filter = ident;
+	newScore.case_sensitive = CASE_INSENSITIVE;
+	newScore.negative = false;
+	scores.push_back(newScore);
+      }
+      else {
+	newScore.score = score;
+	newScore.filter = ident;
+	newScore.case_sensitive = CASE_INSENSITIVE;
+	newScore.negative = true;
+	scores.push_back(newScore);
+      }
+    }
+    else if (cmpNoCase(keyw, "MAXSIZE_SCORE") == 0) {
+      int size = atoi(ident.c_str());
+      if (size > 0 && currOp == "=") {
+	size_score.score = score;
+	size_score.size = size;
+      }
+      else if (size != 0) {
+	showErrorParameter(keyw, currOp, ident);
+	throw MalformedPrefsFile();
+      }
+    }
+    else {
+      showErrorParameter(keyw, currOp, ident);
+      throw MalformedPrefsFile();
+    }
+  }
+
 
   // Precompile all DENY and ALLOW Regular Expressions
   int Preferences::precompileExpressions(void) {
     vector<filterInfo>::iterator curFilter = getFilters()->begin();
     vector<friendInfo>::iterator curFriend = getFriends()->begin();
+    vector<scoreInfo>::iterator curScore = getScores()->begin();
     int error = 0;
 
     try {
@@ -300,9 +378,9 @@ namespace pref {
 	    size_t errLength = regExp.error(error, &curFilter->cFilter, (char*)NULL, (size_t)0);
 	    char errBuf[errLength + 1];
 	    regExp.error(error, &curFilter->cFilter, errBuf, sizeof errBuf);
-	    cerr << PACKAGE << _(": Could not pre-compile this Regular Expression '") << curFilter->filter << "'." << endl;
-	    cerr << PACKAGE << _(": Internal error code from the Regular Expression library: ") << error << endl;
-	    cerr << PACKAGE << ": " << errBuf << endl;
+	    cerr << PACKAGE_NAME << _(": Could not pre-compile this Regular Expression '") << curFilter->filter << "'." << endl;
+	    cerr << PACKAGE_NAME << _(": Internal error code from the Regular Expression library: ") << error << endl;
+	    cerr << PACKAGE_NAME << ": " << errBuf << endl;
 	    return -1;
 	  }
 	}
@@ -311,9 +389,9 @@ namespace pref {
 	    size_t errLength = regExp.error(error, &curFilter->cFilter, (char*)NULL, (size_t)0);
 	    char errBuf[errLength + 1];
 	    regExp.error(error, &curFilter->cFilter, errBuf, sizeof errBuf);
-	    cerr << PACKAGE << _(": Could not pre-compile this Regular Expression '") << curFilter->filter << "'." << endl;
-	    cerr << PACKAGE << _(": Internal error code from the Regular Expression library: ") << error << endl;
-	    cerr << PACKAGE << ": " << errBuf << endl;
+	    cerr << PACKAGE_NAME << _(": Could not pre-compile this Regular Expression '") << curFilter->filter << "'." << endl;
+	    cerr << PACKAGE_NAME << _(": Internal error code from the Regular Expression library: ") << error << endl;
+	    cerr << PACKAGE_NAME << ": " << errBuf << endl;
 	    return -1;
 	  }
 	}
@@ -330,9 +408,9 @@ namespace pref {
 	    size_t errLength = regExp.error(error, &curFriend->cFilter, (char*)NULL, (size_t)0);
 	    char errBuf[errLength + 1];
 	    regExp.error(error, &curFriend->cFilter, errBuf, sizeof errBuf);
-	    cerr << PACKAGE << _(": Could not pre-compile this Regular Expression '") << curFriend->filter << "'." << endl;
-	    cerr << PACKAGE << _(": Internal error code from the Regular Expression library: ") << error << endl;
-	    cerr << PACKAGE << ": " << errBuf << endl;
+	    cerr << PACKAGE_NAME << _(": Could not pre-compile this Regular Expression '") << curFriend->filter << "'." << endl;
+	    cerr << PACKAGE_NAME << _(": Internal error code from the Regular Expression library: ") << error << endl;
+	    cerr << PACKAGE_NAME << ": " << errBuf << endl;
 	    return -1;
 	  }
 	}      
@@ -341,9 +419,9 @@ namespace pref {
 	    size_t errLength = regExp.error(error, &curFriend->cFilter, (char*)NULL, (size_t)0);
 	    char errBuf[errLength + 1];
 	    regExp.error(error, &curFriend->cFilter, errBuf, sizeof errBuf);
-	    cerr << PACKAGE << _(": Could not pre-compile this Regular Expression '") << curFriend->filter << "'." << endl;
-	    cerr << PACKAGE << _(": Internal error code from the Regular Expression library: ") << error << endl;
-	    cerr << PACKAGE << ": " << errBuf << endl;
+	    cerr << PACKAGE_NAME << _(": Could not pre-compile this Regular Expression '") << curFriend->filter << "'." << endl;
+	    cerr << PACKAGE_NAME << _(": Internal error code from the Regular Expression library: ") << error << endl;
+	    cerr << PACKAGE_NAME << ": " << errBuf << endl;
 	    return -1;
 	  }
 	}
@@ -351,6 +429,37 @@ namespace pref {
 	curFriend++;
       }
       
+
+      // Compile scored filters
+      while ( curScore != getScores()->end() ) {
+	error = 0;
+	
+	if ( ((getIcase() == false)  && (curScore->case_sensitive == CASE_DEFAULT)) || (curScore->case_sensitive == CASE_INSENSITIVE) ) {
+	  if ((error = regExp.comp(&curScore->cFilter, curScore->filter, REG_ICASE | getRegType() | getRegNewline())) != 0) {
+	    size_t errLength = regExp.error(error, &curScore->cFilter, (char*)NULL, (size_t)0);
+	    char errBuf[errLength + 1];
+	    regExp.error(error, &curScore->cFilter, errBuf, sizeof errBuf);
+	    cerr << PACKAGE_NAME << _(": Could not pre-compile this Regular Expression '") << curScore->filter << "'." << endl;
+	    cerr << PACKAGE_NAME << _(": Internal error code from the Regular Expression library: ") << error << endl;
+	    cerr << PACKAGE_NAME << ": " << errBuf << endl;
+	    return -1;
+	  }
+	}      
+	else if ( ((getIcase() == true) && (curScore->case_sensitive == CASE_DEFAULT)) || (curScore->case_sensitive == CASE_SENSITIVE) ) {
+	  if ((error = regExp.comp(&curScore->cFilter, curScore->filter, getRegType() | getRegNewline())) != 0) {
+	    size_t errLength = regExp.error(error, &curScore->cFilter, (char*)NULL, (size_t)0);
+	    char errBuf[errLength + 1];
+	    regExp.error(error, &curScore->cFilter, errBuf, sizeof errBuf);
+	    cerr << PACKAGE_NAME << _(": Could not pre-compile this Regular Expression '") << curScore->filter << "'." << endl;
+	    cerr << PACKAGE_NAME << _(": Internal error code from the Regular Expression library: ") << error << endl;
+	    cerr << PACKAGE_NAME << ": " << errBuf << endl;
+	    return -1;
+	  }
+	}
+	
+	curScore++;
+      }
+
       return 0;
     }
     catch (...) {
@@ -437,6 +546,10 @@ namespace pref {
     return test;
   }
 
+  
+  void Preferences::setTestMode(bool newMode) {
+    test = newMode;
+  }
 
   int Preferences::getMaxLineLength(void) {
     return maxLineLength;
@@ -455,27 +568,33 @@ namespace pref {
   }
 
 
+  // Return vector with all compiled scores
+  vector<scoreInfo>* Preferences::getScores(void) {
+    return &scores;
+  }
+
+
   // Shows an error if a keyword is deprecated
   void Preferences::showErrorDeprecated(const string& dKeyword) {
-    cerr << PACKAGE << _(": Deprecated keyword '") << dKeyword << _("' in '") << file << "'." << endl;;
+    cerr << PACKAGE_NAME << _(": Deprecated keyword '") << dKeyword << _("' in '") << file << "'." << endl;;
     
     if (dKeyword == "ICASE")
-      cerr << PACKAGE << _(": Instead use the keyword 'REG_CASE' from now on.") << endl;
+      cerr << PACKAGE_NAME << _(": Instead use the keyword 'REG_CASE' from now on.") << endl;
     else if (dKeyword == "MODE")
-      cerr << PACKAGE << _(": Instead use the keyword 'VERBOSE' from now on.") << endl;
+      cerr << PACKAGE_NAME << _(": Instead use the keyword 'VERBOSE' from now on.") << endl;
 
-    cerr << PACKAGE << _(": Consult the mailfilterrc(5) man page for further details.") << endl;
+    cerr << PACKAGE_NAME << _(": Consult the mailfilterrc(5) man page for further details.") << endl;
   }
 
 
   // Shows an error if a keyword got wrong arguments
   void Preferences::showErrorParameter(const string& keyword, const string& currOp, const string& parameter) {
-    cerr << PACKAGE << _(": A keyword in '") << file << _("' contains invalid parameters.") << endl;
-    cerr << PACKAGE << ": '" <<  keyword << currOp << parameter << "'" << endl;
+    cerr << PACKAGE_NAME << _(": A keyword in '") << file << _("' contains invalid parameters.") << endl;
+    cerr << PACKAGE_NAME << ": '" <<  keyword << currOp << parameter << "'" << endl;
   }
   void Preferences::showErrorParameter(const string& keyword, const string& currOp, int value) {
-    cerr << PACKAGE << _(": A keyword in '") << file << _("' contains invalid parameters.") << endl;
-    cerr << PACKAGE << ": '" <<  keyword << currOp << value << "'" << endl;
+    cerr << PACKAGE_NAME << _(": A keyword in '") << file << _("' contains invalid parameters.") << endl;
+    cerr << PACKAGE_NAME << ": '" <<  keyword << currOp << value << "'" << endl;
   }
 
   // Returns true if Mailfilter should delete duplicates, false otherwise
@@ -494,4 +613,18 @@ namespace pref {
     return timeOut;
   }
 
+
+  int Preferences::getHighscore(void) {
+    return highscore;
+  }
+
+  
+  void Preferences::setHighscore(int j) {
+    highscore = j;
+  }
+
+  Size_score Preferences::getMaxSizeScore(void) {
+    return size_score;
+  }
+  
 }
