@@ -69,7 +69,7 @@ namespace rfc {
 	}
 	
 	// Read field description
-	while (head[i] != ':') {
+	while (head[i] != ':' && i < (int)head.length()) {
 	  curDescr << head[i]; i++;
 	  
 #ifdef DEBUG
@@ -82,6 +82,9 @@ namespace rfc {
 	cout << ": ";
 #endif
 	
+	if (i >= (int)head.length())
+	  continue;
+	
 	// Ignore possible white spaces in between field-name and field-body
 	// (Necessary cause some mailers and/or clients do insert them.)
 	while (head[i] == ' ') i++;
